package gssim.db.dao.data;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @author Jarek Szymczak
 * 
 */
public class Unit {

	protected Integer PK;
	protected Integer resourceId;
	protected String type;
	protected String name;
	protected Integer amount;
	protected Double allocationLoad;
	protected Double reservationLoad;
	protected List<UnitDetails> unitDetails = new ArrayList<UnitDetails>();

	public Integer getPK() {
		return PK;
	}

	/**
	 * Sets the primary key in the unit class and foreign keys in unit details
	 * list.
	 * 
	 * @param pK
	 *            primary key
	 */
	public void setPK(Integer pK) {
		PK = pK;
		if (unitDetails.size() > 0)
			for (UnitDetails unitDetail : unitDetails) {
				unitDetail.setFkUnitId(pK);
			}
	}

	public Integer getResourceId() {
		return resourceId;
	}

	public void setResourceId(Integer resourceId) {
		this.resourceId = resourceId;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Integer getAmount() {
		return amount;
	}

	public void setAmount(Integer amount) {
		this.amount = amount;
	}

	public Double getAllocationLoad() {
		return allocationLoad;
	}

	public void setAllocationLoad(Double allocationLoad) {
		this.allocationLoad = allocationLoad;
	}

	public Double getReservationLoad() {
		return reservationLoad;
	}

	public void setReservationLoad(Double reservationLoad) {
		this.reservationLoad = reservationLoad;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + PK;
		long temp;
		temp = Double.doubleToLongBits(allocationLoad);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + amount;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		temp = Double.doubleToLongBits(reservationLoad);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + resourceId;
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		return result;
	}

	public void addUnitDetails(UnitDetails unitDetail) {
		unitDetails.add(unitDetail);
	}

	/**
	 * Gets the unit details on list.
	 * 
	 * @param accUnitDetails
	 *            the list, on which unit details are copied (the list should be
	 *            initialized)
	 */
	public void getUnitDetailsOnList(List<UnitDetails> accUnitDetails) {
		if (accUnitDetails == null)
			return;
		for (UnitDetails unitDetail : unitDetails) {
			accUnitDetails.add(unitDetail);
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Unit other = (Unit) obj;
		if (PK != other.PK)
			return false;
		if (Double.doubleToLongBits(allocationLoad) != Double
				.doubleToLongBits(other.allocationLoad))
			return false;
		if (amount != other.amount)
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (Double.doubleToLongBits(reservationLoad) != Double
				.doubleToLongBits(other.reservationLoad))
			return false;
		if (resourceId != other.resourceId)
			return false;
		if (type == null) {
			if (other.type != null)
				return false;
		} else if (!type.equals(other.type))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Unit [PK=" + PK + ", resourceId=" + resourceId + ", type="
				+ type + ", name=" + name + ", amount=" + amount
				+ ", allocationLoad=" + allocationLoad + ", reservationLoad="
				+ reservationLoad + "]";
	}
}
