package gssim.db.mysql;

import gssim.db.dao.data.ExperimentStats;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class ExperimentStatsDAO extends AbstractDAO implements
		gssim.db.dao.ExperimentStatsDAO {

	protected ExperimentStatsDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(ExperimentStatsDAO.class);
		setRelationName("experiment_stats");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		return null;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM experiment_stats WHERE"
				+ " (id = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO experiment_stats VALUES(?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM experiment_stats WHERE id = ? AND name = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		return null;
	}

	@Override
	public boolean delete(ExperimentStats obj, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public void fillObjectFromResultSet(ExperimentStats obj, ResultSet result)
			throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt,
			ExperimentStats obj, int offset) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public List<ExperimentStats> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<ExperimentStats> list = new ArrayList<ExperimentStats>();

		stmt.setInt(1, finder.getInteger("id"));
		stmt.setInt(2, finder.getInteger("id"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			ExperimentStats obj = new ExperimentStats();

			Integer id = result.getIntObj("id");
			String name = result.getString("name");
			Double value = result.getDoubleObj("value");

			obj.setId(id);
			obj.setName(name);
			obj.setValue(value);

			list.add(obj);
		}

		return list;
	}

	@Override
	public int insert(ExperimentStats obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		stmt.setInt(1, obj.getId());
		stmt.setString(2, obj.getName());
		stmt.setDouble(3, obj.getValue());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("insert");
		} else
			logError("insert", result, obj);

		return result;
	}

	@Override
	public int[] insertBatch(List<ExperimentStats> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			stmt.setInt(1, objList.get(i).getId());
			stmt.setString(2, objList.get(i).getName());
			stmt.setDouble(3, objList.get(i).getValue());
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {
			if (commit)
				commit();

			logSuccess("batch insert", j);

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public ExperimentStats select(ExperimentStats obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getId());
		stmt.setString(2, obj.getName());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			obj.setValue(result.getDoubleObj("value"));
			return obj;
		}

		// no reservation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");
	}

	@Override
	public boolean update(ExperimentStats obj, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

}